local ADDONNAME, BestScrapMeltdown = ...

local _G = _G
local pairs, tonumber, strmatch, format =
      pairs, tonumber, strmatch, format
local GetQuestID, GetItemCount, GetNumQuestChoices, GetQuestItemLink, CreateFrame =
      GetQuestID, GetItemCount, GetNumQuestChoices, GetQuestItemLink, CreateFrame
local QuestInfoRewardsFrame, AutoCastShine_AutoCastStart, AutoCastShine_AutoCastStop =
      QuestInfoRewardsFrame, AutoCastShine_AutoCastStart, AutoCastShine_AutoCastStop


local QUEST_IDS = {
    [38175] = true, -- Alliance
    [38188] = true, -- Horde
}

local ARMOUR_TOKEN = 120301
local WEAPON_TOKEN = 120302

local WEAPON_ITEM_IDS = {
    [114128] = 3,
    [114129] = 6,
    [114131] = 9,
}
local ARMOUR_ITEM_IDS = {
    [114745] = 3,
    [114808] = 6,
    [114822] = 9,
}

do-- Event Handling
    local eventFrame = CreateFrame("Frame")
    eventFrame:SetScript("OnEvent", function(frame, event, ...)
        if BestScrapMeltdown[event] then
            BestScrapMeltdown[event](BestScrapMeltdown, event, ...)
        end
    end)
    eventFrame:RegisterEvent("QUEST_COMPLETE")

    function BestScrapMeltdown:RegisterEvent(event)
        eventFrame:RegisterEvent(event)
    end
    function BestScrapMeltdown:UnregisterEvent(event)
        eventFrame:UnregisterEvent(event)
    end
end


function BestScrapMeltdown:QUEST_FINISHED()
    self:UnregisterEvent("QUEST_FINISHED")
    self:UnregisterEvent("BAG_UPDATE")
end

function BestScrapMeltdown:BAG_UPDATE()
    self:QUEST_COMPLETE()
end

function BestScrapMeltdown:QUEST_COMPLETE()
    self:HideRewardHighlights()

    if QUEST_IDS[GetQuestID()] then
        self:RegisterEvent("QUEST_FINISHED")

        local maxArmouriLvL = 0
        local maxWeaponiLvL = 0

        for itemID, value in pairs(WEAPON_ITEM_IDS) do
            maxWeaponiLvL = maxWeaponiLvL + value * GetItemCount(itemID, true)
        end
        for itemID, value in pairs(ARMOUR_ITEM_IDS) do
            maxArmouriLvL = maxArmouriLvL + value * GetItemCount(itemID, true)
        end

        self:RegisterEvent("BAG_UPDATE")

        --[===[@debug@
        print("maxArmouriLvL: ", maxArmouriLvL)
        print("maxWeaponiLvL: ", maxWeaponiLvL)
        --@end-debug@]===]

        if maxArmouriLvL < maxWeaponiLvL then
            for i = 1, GetNumQuestChoices() do
                local link = GetQuestItemLink("choice", i)
                local itemID = tonumber( strmatch(link, "item:(%d+)") )
                if itemID == ARMOUR_TOKEN then
                    self:SetRewardHighlight(i)
                    break
                end
            end
        elseif maxWeaponiLvL < maxArmouriLvL then
            for i = 1, GetNumQuestChoices() do
                local link = GetQuestItemLink("choice", i)
                local itemID = tonumber( strmatch(link, "item:(%d+)") )
                if itemID == WEAPON_TOKEN then
                    self:SetRewardHighlight(i)
                    break
                end
            end
        else
            for i = 1, GetNumQuestChoices() do
                local link = GetQuestItemLink("choice", i)
                local itemID = tonumber( strmatch(link, "item:(%d+)") )

                if itemID == ARMOUR_TOKEN then
                    self:SetRewardHighlight(i)
                end
                if itemID == WEAPON_TOKEN then
                    self:SetRewardHighlight(i)
                end
            end
        end
    end
end

do-- Reward Highlight
    local rewardHighlightFrames = {}
    local function rewardHighlightFrame_OnHide(self)
        AutoCastShine_AutoCastStop(self)
    end

    -- Original code: Quester from Nevcairiel
    function BestScrapMeltdown:SetRewardHighlight(rewardID)
        local frame = rewardHighlightFrames[rewardID]

        if not frame then
            frame = CreateFrame("Frame", "BestScrapMeltdownRewardHighlight"..rewardID, QuestInfoRewardsFrame, "AutoCastShineTemplate")
            frame:SetScript("OnHide", rewardHighlightFrame_OnHide)
            rewardHighlightFrames[rewardID] = frame

            frame:ClearAllPoints()
            frame:SetAllPoints( _G[ format("QuestInfoRewardsFrameQuestInfoItem%dIconTexture", rewardID) ] )
        end

        frame:Show()
        AutoCastShine_AutoCastStart(frame)
    end

    function BestScrapMeltdown:HideRewardHighlights()
        for _, frame in pairs(rewardHighlightFrames) do
            frame:Hide()
        end
    end
end
